"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const mysql = require("mysql");
const sleep_es6_1 = require("sleep-es6");
const columns_1 = require("./util/columns");
const insert_1 = require("./util/insert");
class DB {
    constructor(config) {
        this.config = config;
        this.connect();
    }
    close() {
        return __awaiter(this, void 0, void 0, function* () {
            yield new Promise((resolve, reject) => {
                this.connection.end((err) => {
                    if (err) {
                        reject(err);
                    }
                    resolve();
                });
            });
        });
    }
    query(sql, values) {
        values = values || [];
        return new Promise((resolve, reject) => {
            const query = this.connection.query(sql, values, (err, results) => {
                if (err) {
                    reject(err);
                    return;
                }
                resolve(results);
            });
            if (this.config.logging) {
                this.log(query.sql);
            }
        });
    }
    insert(tableName, rows, opts = { ignoreDuplicate: false, onDuplicateKeyUpdateAll: false, ignoreNullableValues: false }) {
        return __awaiter(this, void 0, void 0, function* () {
            let schemaName;
            if (!this.config.database) {
                const p = tableName.split(".");
                if (p.length === 1) {
                    throw new Error("Schema should be setted");
                }
                schemaName = p[0];
                tableName = p[1];
            }
            else {
                schemaName = this.config.database;
            }
            const columns = yield columns_1.default(this)(schemaName, tableName);
            const fields = insert_1.getFieldsFromList(rows, columns);
            const forInserting = rows.map((row) => {
                const cols = [];
                for (const fieldName in fields) {
                    if (typeof (row[fieldName]) === "undefined" || row[fieldName] === null) {
                        if (!fields[fieldName].isNullable && !opts.ignoreNullableValues) {
                            throw new Error("Empty value for non-nullable field " + fieldName);
                        }
                        cols.push(null);
                    }
                    else {
                        cols.push(insert_1.prepareValue(row[fieldName], fields[fieldName].type, fieldName, row));
                    }
                }
                return cols;
            });
            const fieldsNames = Object.keys(fields);
            return this.query(`insert${opts.ignoreDuplicate ? " ignore" : ""} into ` + tableName + "(`" +
                fieldsNames.join("`,`")
                + "`) values ?" + (opts.onDuplicateKeyUpdateAll ?
                ` on duplicate key update ` +
                    fieldsNames.map((name) => "`" + name + "` = VALUES(`" + name + "`)").join(", ")
                : ""), [forInserting]);
        });
    }
    beginTransaction() {
        return __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve, reject) => {
                this.connection.beginTransaction((err) => {
                    if (err) {
                        reject(err);
                        return;
                    }
                    resolve();
                });
            });
        });
    }
    commit() {
        return __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve, reject) => {
                this.connection.commit((err) => {
                    if (err) {
                        reject(err);
                        return;
                    }
                    resolve();
                });
            });
        });
    }
    rollback() {
        return __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve) => {
                this.connection.rollback(() => {
                    resolve();
                });
            });
        });
    }
    log(...args) {
        console.log.apply(console, args);
    }
    connect() {
        return __awaiter(this, void 0, void 0, function* () {
            this.connection = mysql.createConnection(this.config);
            this.connection.connect((err) => __awaiter(this, void 0, void 0, function* () {
                if (err) {
                    this.log("Connection error:", err);
                    yield sleep_es6_1.default(1000);
                    this.connect();
                    return;
                }
            }));
            this.connection.on("error", (err) => __awaiter(this, void 0, void 0, function* () {
                this.log("Connection error:", err);
                yield sleep_es6_1.default(1000);
                this.connect();
            }));
        });
    }
}
exports.default = DB;
