"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
jest.unmock("./DB");
jest.unmock("./util/insert");
jest.unmock("./util/ColumnDataType");
jest.mock("./util/columns", () => {
    return {
        default: () => jest.fn().mockImplementation((schemaName, tableName) => {
            if (schemaName === "schema" && tableName === "table1") {
                return {
                    id: { type: "int" },
                    name: { type: "longtext" },
                    field1: { type: "string", isNullable: true },
                    json1: { type: "json", isNullable: true },
                };
            }
            throw new Error("Unknoww table");
        }),
    };
});
const mockMysql = require("./../__mocks__/mysql");
const DB_1 = require("./DB");
const connection = {
    query: jest.fn(),
};
mockMysql.createConnection.mockReturnValueOnce(connection);
connection.query.mockImplementation((_, __, cb) => {
    cb(null, "query1");
});
const db = new DB_1.default({
    user: "root",
    password: "123",
});
beforeEach(() => {
    connection.query.mockClear();
});
it("create", () => {
    expect(db.connection).toBe(connection);
});
it("insert", () => __awaiter(this, void 0, void 0, function* () {
    const res = yield db.insert("schema.table1", [{ id: 1, name: "test1" }]);
    expect(res).toBe("query1");
    expect(connection.query.mock.calls).toEqual([[
            "insert into table1(`id`,`name`) values ?",
            [[[1, "test1"]]],
            jasmine.any(Function),
        ]]);
}));
it("insert on duplicate key update", () => __awaiter(this, void 0, void 0, function* () {
    yield db.insert("schema.table1", [{ id: 1, name: "test1" }], { onDuplicateKeyUpdateAll: true });
    expect(connection.query.mock.calls).toEqual([[
            "insert into table1(`id`,`name`) values ? on duplicate key update `id` = VALUES(`id`), `name` = VALUES(`name`)",
            [[[1, "test1"]]],
            jasmine.any(Function),
        ]]);
}));
it("insert with excesses fields", () => __awaiter(this, void 0, void 0, function* () {
    yield db.insert("schema.table1", [
        { id: 1, name: "test1", e1: true, field1: "str1" },
        { name: { test: "x" }, id: 2, ee2: "v", json1: { temp: 8 } },
    ]);
    expect(connection.query.mock.calls[0][0]).toBe("insert into table1(`id`,`name`,`field1`,`json1`) values ?");
    expect(connection.query.mock.calls[0][1][0]).toEqual([
        [1, "test1", "str1", null],
        [2, "{\"test\":\"x\"}", null, "{\"temp\":8}"]
    ]);
}));
