"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
class HttpClient {
    constructor(config) {
        this.config = config;
    }
    query(q, params) {
        return __awaiter(this, void 0, void 0, function* () {
            const res = yield fetch(this.config.address + "/query", {
                method: "POST",
                headers: {
                    "Content-Type": "application/json",
                },
                body: JSON.stringify({
                    query: q,
                    params,
                }),
            });
            if (res.status !== 200) {
                throw new Error("Invalid request, status " + res.status + ": " + res.statusText
                    + ", " + (yield res.text()));
            }
            return yield res.json();
        });
    }
    insert(tableName, rows, opts = { ignoreDuplicate: false, onDuplicateKeyUpdateAll: false, ignoreNullableValues: false }) {
        return __awaiter(this, void 0, void 0, function* () {
            const res = yield fetch(this.config.address + "/insert", {
                method: "POST",
                headers: {
                    "Content-Type": "application/json",
                },
                body: JSON.stringify({
                    tableName,
                    rows,
                    opts,
                }),
            });
            if (res.status !== 200) {
                throw new Error("Invalid request, status " + res.status + ": " + res.statusText
                    + ", " + (yield res.text()));
            }
            return yield res.json();
        });
    }
}
exports.default = HttpClient;
