"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
class HttpProxy {
    constructor(db, config) {
        this.db = db;
        this.config = config;
    }
    attach(app) {
        return __awaiter(this, void 0, void 0, function* () {
            const bodyParser = yield Promise.resolve().then(function () { return require("body-parser"); });
            app.use("/query", (req, res, next) => {
                res.header("Access-Control-Allow-Origin", "*");
                res.header("Access-Control-Allow-Headers", "Content-Type, Authorization, Content-Length, X-Requested-With");
                if (req.method === "OPTIONS") {
                    res.sendStatus(200);
                }
                else {
                    next();
                }
            });
            app.post("/insert", bodyParser.json(), (req, res) => __awaiter(this, void 0, void 0, function* () {
                try {
                    const { tableName, rows, opts } = req.body;
                    const result = yield this.db.insert(tableName, rows, opts);
                    res.send(result);
                }
                catch (e) {
                    res.writeHead(400, "Database error");
                    res.write(e.toString());
                    res.end();
                }
            }));
            app.get("/query", (req, res) => __awaiter(this, void 0, void 0, function* () {
                const p = req.query.params ?
                    JSON.parse(req.query.params) : [];
                try {
                    const result = yield this.db.query(req.query.query, p);
                    res.send(result);
                }
                catch (e) {
                    res.writeHead(400, "Database error");
                    res.write(e.toString());
                    res.end();
                }
            }));
            app.post("/query", bodyParser.json(), (req, res) => __awaiter(this, void 0, void 0, function* () {
                try {
                    const result = yield this.db.query(req.body.query, req.body.params ? req.body.params : []);
                    res.send(result);
                }
                catch (e) {
                    res.writeHead(400, "Database error");
                    res.write(e.toString());
                    res.end();
                }
            }));
        });
    }
    start() {
        return __awaiter(this, void 0, void 0, function* () {
            const express = yield Promise.resolve().then(function () { return require("express"); });
            const app = express();
            this.attach(app);
            yield new Promise((resolve, reject) => {
                this.server = app.listen(this.config.port || 7001, (err) => {
                    if (err) {
                        reject(err);
                        return;
                    }
                    resolve();
                });
            });
        });
    }
    stop() {
        return __awaiter(this, void 0, void 0, function* () {
            yield new Promise((resolve, reject) => {
                if (this.server) {
                    this.server.close((err) => {
                        if (err) {
                            reject(err);
                            return;
                        }
                        resolve();
                    });
                }
                else {
                    resolve();
                }
            });
        });
    }
}
exports.default = HttpProxy;
