import { Express } from "express";
import { Server } from "http";
import DB from "./DB";
export interface IProxyConfig {
    port?: number;
}
class HttpProxy {
    protected server?: Server;
    constructor(protected db: DB, protected config: IProxyConfig) { }
    public async attach(app: Express) {
        const bodyParser = await import("body-parser");
        app.use("/query", (req, res, next) => {
            res.header("Access-Control-Allow-Origin", "*");
            res.header("Access-Control-Allow-Headers", "Content-Type, Authorization, Content-Length, X-Requested-With");
            if (req.method === "OPTIONS") {
                res.sendStatus(200);
            } else {
                next();
            }
        });
        app.post("/insert", bodyParser.json(), async (req, res) => {
            try {
                const { tableName, rows, opts } = req.body;
                const result = await this.db.insert(tableName, rows, opts);
                res.send(result);
            } catch (e) {
                res.writeHead(400, "Database error");
                res.write(e.toString());
                res.end();
            }
        });
        app.get("/query", async (req, res) => {
            const p = req.query.params ?
                JSON.parse(req.query.params) : [];
            try {
                const result = await this.db.query(req.query.query, p);
                res.send(result);
            } catch (e) {
                res.writeHead(400, "Database error");
                res.write(e.toString());
                res.end();
            }
        });
        app.post("/query", bodyParser.json(), async (req, res) => {
            try {
                const result = await this.db.query(req.body.query, req.body.params ? req.body.params : []);
                res.send(result);
            } catch (e) {
                res.writeHead(400, "Database error");
                res.write(e.toString());
                res.end();
            }
        });
    }
    public async start() {
        const express = await import("express");
        const app = express();
        this.attach(app);
        await new Promise((resolve, reject) => {
            this.server = app.listen(this.config.port || 7001, (err: any) => {
                if (err) {
                    reject(err);
                    return;
                }
                resolve();
            });
        });
    }
    public async stop() {
        await new Promise((resolve, reject) => {
            if (this.server) {
                this.server.close((err: any) => {
                    if (err) {
                        reject(err);
                        return;
                    }
                    resolve();
                });
            } else {
                resolve();
            }
        });
    }
}
export default HttpProxy;
